<?php

namespace App\Controller;

use App\Exception\ItemException as ItemException;
use App\Exception\IntegrationException as IntegrationException;
use App\Exception\ParamsException as ParamsException;
use App\Exception\SqlException as SqlException;
use App\Exception\CancelException as CancelException;
use App\Exception\NetworkException as NetworkException;
use App\Exception\GeneralErrorException as GeneralErrorException;


use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

class WebService {

    const FAILOVER_EMAIL = "daniel.fonseca@teknisa.com";
    //const FAILOVER_EMAIL = "daniel.santos@teknisa.com";

    protected $connection;
    protected $orderService;
    protected $consumerService;
    protected $productService;
    protected $paymentService;
    protected $util;
    
    private $mailList;
    
    const API_ANT_BASE_URL  = "https://api-parceiros.anota.ai/";
    const GEOCODE_URL       = "https://maps.google.com/maps/api/geocode/json?key=AIzaSyBfblcHNG4xb8lcwzD5sUvQ-dXIuY3QbNs=";

    public function __construct(\Doctrine\DBAL\Connection $connection,
                                \App\Service\OrderService $orderService, 
                                \App\Service\ConsumerService $consumerService,
                                \App\Service\ProductService $productService, 
                                \App\Service\PaymentService $paymentService,
                                \App\Util\Util $util) {

        $this->connection       = $connection;
        $this->orderService     = $orderService;
        $this->consumerService  = $consumerService;
        $this->productService   = $productService;
        $this->paymentService   = $paymentService;
        $this->util             = $util;
    }

    public function showVersion(Request $request, Response $response) {
        $sql        = "SELECT CDFILIAL FROM FILIAL";
        $result     = $this->connection->fetchAll($sql);

        $versionInfo = $this->getVersionInfo();

        return $response->withJson([
            'version'     => $versionInfo['version'],
            'revision'    => $versionInfo['revision'],
            'date'        => $versionInfo['date'],
            'branchs'     => count($result)
            //'database'    => ''//$this->connection->getParams()
        ]);
    }
    
    private function getVersionInfo() {
        $file = file_get_contents(__DIR__ . "/../../version.json");
        return json_decode($file, true);
    }

    /**
    * Função de entrada dos pedidos provenientes da Anota Aí, os pedidos chegam pelo WebHook respectivo
    *
    * @param Request $request
    * @param Response $response
    * @throws ParamsException Exceção de parâmetros inválidos ou faltantes.
    * @throws ItemException Exceção de parâmetros inválidos nos itens do pedido.
    * @throws SqlException Exceção de erros SQL.
    * @throws Exception Exceções gerais.
    * @return void
    */
    public function runWebService(Request $request, Response $response)
    {
        $params = $request->getParsedBody();

        // $this->debugLog($params);

        if ($params["auth"] != "81743A8E8D41AF82A4807097CDF4BEC2DDDB9E6334BECCFED4D46FC783A8CA2E") {
            $response->withStatus(400);
            return 0;
        }
            
        $errorMessage = "";
        $rawOrder     = "";
        $writeEmail   = true;
        
        $configuration = $this->util->getConfiguration();
        if(empty($configuration)) {
            $response->withStatus(200);
            return 0;
        }

        $ufTimezone = (isset($configuration['IDUFTIMEZONE']) && !empty($configuration['IDUFTIMEZONE'])) ? $configuration['IDUFTIMEZONE'] : "SP";
        
        date_default_timezone_set($this->util->getCurrentTimezone($ufTimezone));
        
        if($configuration['IDHABINTE'] == 'N') 
            return $response->withStatus(200);
        
        $this->mailList = $this->util->getMailList();
        $this->connection->beginTransaction();
        
        try {
            $this->mailList = $this->util->getMailList();
            $orderDetails   = $params["anotaAiData"]["info"];

            // $orderId    = $params['event']['data']['order_id']; --DEPRECIADO
            // $appId      = $params['app_id']; --DEPRECIADO
            // $appSecret  = $params['app_secret']; --DEPRECIADO
            // $appShopId  = $params['event']['app_shop_id']; --DEPRECIADO
            $debug          = isset($params['debug']) ? $params['debug'] : false;
            $merchantId     = $orderDetails["merchant"]["id"];
            $orderId        = $orderDetails['id'];
            
            $rawOrder       = $orderDetails;
            $deliveryParams = $this->orderService->getDeliveryMethodParams($merchantId);
    
            if(empty($deliveryParams['IDTOKENINTGERAL']))
                throw new GeneralErrorException("Token geral não encontrado, associado ao id ".$merchantId." do estabelecimento.");
            else{
                $this->util->setAccessToken($deliveryParams['IDTOKENINTGERAL']);
            }    

            $deliveryParams['insertChange'] = ($deliveryParams['IDRETTXENTR'] == "N");
            //$deliveryParams['outsourcedDelivery'] = ($deliveryParams['IDUTLENTRETERC'] == "S");

            $cdFilial = $deliveryParams["CDFILIAL"];
            
            $fileName = ((!empty($cdFilial)) && (!empty($orderId))) ? "{$cdFilial}-{$orderId}-{$this->util->getCurrentDateTime()}.json" : ("INDEFINIDO-" . $this->util->getCurrentDateTime()) . ".json";
            
            $this->util->writeLog('order', $fileName, $orderDetails);

            $integrationDetails = $this->processOrder($cdFilial, $merchantId, $orderId, $orderDetails, $deliveryParams, $debug);

            $branchId = $integrationDetails["CDFILIAL"];
            $sellId  = $integrationDetails["NRVENDAREST"];
            $this->orderService->validateOrder($branchId, $sellId);
            $this->util->writeLog('order_processed', $fileName, $orderDetails);
            
            if(!$debug) {
              $acceptOrder = $this->acceptOrder($orderId);
              $this->util->writeLog('accepted', $fileName, $acceptOrder);
            }
            $this->connection->commit();

            return $response->withStatus(200);

        } catch(ItemException $e) {
            $errorMessage = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            if(empty($fileName)) $fileName = "INDEFINIDO-{$this->util->getCurrentDateTime()}.json";
            
            $this->util->writeLog("order_error", $fileName, $errorMessage);
            $this->connection->rollback();
            if(!$debug){
                $responseReject = $this->rejectOrder($orderId);    
                $this->util->writeLog('rejected', $fileName, $responseReject);
            }
        } catch(ParamsException $e) {
            $errorMessage = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            if(empty($fileName)) $fileName = "INDEFINIDO-{$this->util->getCurrentDateTime()}.json";
            
            $this->util->writeLog("order_error", $fileName, $errorMessage);
            $this->connection->rollback();
            if(!$debug){
                $responseReject = $this->rejectOrder($orderId);    
                $this->util->writeLog('rejected', $fileName, $responseReject);
            }
        } catch(SqlException $e) {
            $errorMessage = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            if(empty($fileName)) $fileName = "INDEFINIDO-{$this->util->getCurrentDateTime()}.json";
            
            $this->util->writeLog("order_error", $fileName, $errorMessage);
            $this->connection->rollback();
            if(!$debug){
                $responseReject = $this->rejectOrder($orderId);    
                $this->util->writeLog('rejected', $fileName, $responseReject);
            }
        } catch(NetworkException $e) {
            $errorMessage = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            if(empty($fileName)) $fileName = "INDEFINIDO-{$this->util->getCurrentDateTime()}.json";
            
            $this->util->writeLog("network", $fileName, $errorMessage);
            $this->connection->rollback();
            if(!$debug){
                $responseReject = $this->rejectOrder($orderId);    
                $this->util->writeLog('rejected', $fileName, $responseReject);
            }
        } catch(IntegrationException $e) {
            $writeEmail = false;
            $errorMessage = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            if(empty($fileName)) $fileName = "INDEFINIDO-{$this->util->getCurrentDateTime()}.json";
            
            $this->util->writeLog("order_error", $fileName, $errorMessage);
            $this->connection->rollback();
            if(!$debug){
                $responseReject = $this->rejectOrder($orderId);    
                $this->util->writeLog('rejected', $fileName, $responseReject);
            }
        } catch(CancelException $e) {
            $errorMessage = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            if(empty($fileName)) $fileName = "INDEFINIDO-{$this->util->getCurrentDateTime()}.json";
            
            $this->util->writeLog("cancelled_error", $fileName, $errorMessage);
            $this->connection->rollback();
        } catch(GeneralErrorException $e) {
            $errorMessage = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            if(empty($fileName)) $fileName = "INDEFINIDO-{$this->util->getCurrentDateTime()}.json";
            
            $this->util->writeLog("error", $fileName, $errorMessage);
            $this->connection->rollback();
            // desligar projeto            
        } catch(\Exception $e) {
            $errorMessage = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            if(empty($fileName)) $fileName = "INDEFINIDO-{$this->util->getCurrentDateTime()}.json";
            
            $this->util->writeLog("error", $fileName, $errorMessage);
            $this->connection->rollback();
            if(!$debug){
                $responseReject = $this->rejectOrder($orderId);    
                $this->util->writeLog('rejected', $fileName, $responseReject);
            }
        } 
        if (!empty($errorMessage)) {
            $cdFilial    = $this->getBranch($merchantId);
            $orderNumber = (isset($orderId) ? 
                        ("$cdFilial-{$orderId}") : ("INDEFINIDO-" . date("H-i-s"))); 
            
            /* ----- DEPRECIADO            
            $this->util->writeLog('order_error', date('H-i-s') . '.txt', $errorMessage);
            */
            
            if($writeEmail) {
                global $CLIENT_NAME;
                $clientName = $CLIENT_NAME;

                $mailBody = file_get_contents(__DIR__ . "/../../templates/defaultMail.html");

                if($debug) $this->mailList = null; 

                $this->util->sendMail((!empty($this->mailList) ? $this->mailList : [self::FAILOVER_EMAIL]), [], 'Integração Teknisa x Anota Aí', $mailBody, [
                    ":INTEGRACAO"    => 'Teknisa x Anota Aí',
                    ":NUMERO_PEDIDO" => $orderNumber,
                    ":NOME_CLIENTE"  => $clientName,
                    ":NOME_SERVIDOR" => "",
                    ":ERRO_MSG"      => $errorMessage,
                    ":JSON_PEDIDO"   => json_encode($rawOrder, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE),
                    ":GERADO_EM"     => date("d/m/Y H:i:s")
                ]);
            }
        }

        return $response->withStatus(500);
    }

    private function debugLog($params){
        $folder      = "/home/developer/workfolder/logs/anotaAi/debugLog";
        $fileName = round(microtime(true)*1000).'_order';
        $currentFile = "$folder/$fileName";
        if(!file_exists($folder)) {
            mkdir($folder, 0775, true);
        }
        file_put_contents($currentFile, json_encode($params, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
    }

    private function processOrder($cdFilial, $merchantId, $orderId, $orderDetails, $deliveryParams, $debug) {
        
        $isDuplicate   = $this->validateDuplication($cdFilial, $orderId);

        if(!$debug && $isDuplicate) 
            throw new IntegrationException("O pedido " . $orderId . " já foi integrado pelo sistema.", 302);
        
        $deliveryFee   = $orderDetails['deliveryFee'];
        $defaultParams = $this->getDefaultParams($cdFilial);
        
        if(isset($deliveryParams["IDUTLVNDPDRPALC"]) && ($deliveryParams["IDUTLVNDPDRPALC"] == "N")){
            if(empty($deliveryParams['CDLOJA'])) {
                throw new ParamsException("Não existe um código de loja associado ao estabelecimento de id <b>{$merchantId}</b> Anota Aí.");
            }

            if(empty($deliveryParams['CDCAIXA'])) {
                throw new ParamsException("Não existe um código de caixa associado ao estabelecimento de id <b>{$merchantId}</b> Anota Aí.");
            }
            
            $defaultParams["CDLOJA"] = $deliveryParams["CDLOJA"];
            $defaultParams["CDCAIXA"] = $deliveryParams["CDCAIXA"];
        }
        
        $idstcomanda   = $defaultParams["IDCONTROPROD"] === "S" ? "1" : "3";
        
        $price           = $orderDetails['total'];
        $orderPrice      = $price;
        $orderPaidInCash = $orderDetails['payments'][0]['code'] ==  'money' ? true : false;
        
        $consumerData = $this->getConsumer($defaultParams['CDCLIENTE'], $defaultParams['CDCONSUMIDOR']);
        
        if(!empty($orderDetails['deliveryAddress']) && !empty($orderDetails['deliveryAddress']['formattedAddress'])) 
            $consumerAddress = $this->getConsumerAddress($orderDetails['deliveryAddress']['formattedAddress']);

        if(!empty($consumerAddress)) 
            $consumerData = $consumerAddress;

        $consumerIdentifier  = !empty($orderDetails['customer']['name']) ? strtoupper($this->util->removeAcentos($orderDetails['customer']['name'])) : '';
        $specialInstructions = !empty($orderDetails['customer']['name']) ? ('CLIENTE: ' . $orderDetails['customer']['name'] . ' ') : '';

        if($orderPaidInCash){
            if(!empty($specialInstructions)) {
        		$specialInstructions .= " - ";
        	}
            
            foreach ($orderDetails['payments'] as $payment) {
                if((isset($payment["changeFor"])) && (!empty($payment["changeFor"]))) {
                    $troco = bcsub($payment["changeFor"], $orderDetails["total"], 2);
                    $specialInstructions .= 'RECEBER R$' . $troco . ' EM DINHEIRO.';
                }else{
                    $specialInstructions .= 'PAGAR R$' . $orderDetails['total'] . ' EM DINHEIRO.';
                }
            }    
	
        }
        
        if(!empty($specialInstructions)) {
        	$specialInstructions = strtoupper($this->util->removeAcentos($specialInstructions));
        }

        $nrvendarest    = $this->saveVendarest($cdFilial, $defaultParams['CDLOJA'], $defaultParams['NRMESA'], $defaultParams['CDVENDEDOR'], $defaultParams['CDOPERADOR'], $defaultParams['CDCLIENTE'], $defaultParams['CDCONSUMIDOR'], $defaultParams['SGESTADO']);
        
        if($orderDetails["type"] == "TAKE") 
            $dlvMethod = "S"; // Retirar no estabelecimento
        elseif($orderDetails["type"] == "LOCAL") 
            $dlvMethod = "C"; // Consumir no local
        else 
            $dlvMethod = "N"; // Delivery
        
        $nrcomanda      = $this->saveComandaven($cdFilial, $nrvendarest, $orderDetails['id'], $defaultParams['CDLOJA'], $idstcomanda, $deliveryFee, $orderId, $consumerData, $specialInstructions, $consumerIdentifier, $dlvMethod);
        
        $defaultParams["NRSEQPRODCUP"] = $this->util->newCode("ITCOMANDAVEN".$defaultParams["CDFILIAL"], 10);
        $products     = [];
        $ITCOMANDAVEN_SEQ = 0;

        foreach ($orderDetails["items"] as $item) {
            $processedProduct = $this->productService->processProduct($defaultParams, $item, $nrvendarest, $nrcomanda, $ITCOMANDAVEN_SEQ);
            if (is_array($processedProduct) && !isset($processedProduct["CDPRODUTO"]))
                $products = array_merge($products, $processedProduct);
            else
                array_push($products, $processedProduct);
        }

        if(empty($products)) {
            throw new \Exception("Ocorreu uma falha na validação de produtos.");
        }

        foreach ($products as $product) {
            if(empty($product['VRPRECCOMVEN']) || (bccomp($product['VRPRECCOMVEN'], "0", 2) <= 0)) {
                throw new \Exception("O produto ({$product['CDPRODUTO']}) possui preço inferior a 0.01 centavo. Valor: {$product['VRPRECCOMVEN']}", 191);
            }
            
            if(!empty($product['VRDESCCOMVEN']) && bccomp($product['VRDESCCOMVEN'], $product['VRPRECCOMVEN'], 2) >= 0) {
                throw new \Exception("O produto ({$product['CDPRODUTO']}) possui DESCONTO superior ao VALOR: Desconto -> {$product['VRDESCCOMVEN']} Valor -> {$product['VRPRECCOMVEN']}", 192);
            }
            
            $this->productService->insertProduct(array_merge($product, $defaultParams));
        }
        
        $payments = $orderDetails['payments'];
        $this->saveMovcaixadlv($deliveryParams, $orderDetails, $cdFilial, $nrvendarest, $orderPrice, $orderPaidInCash, $defaultParams['CDCLIENTE'], $defaultParams['CDCONSUMIDOR'], $payments);

        return [
          "CDFILIAL"    => $cdFilial,
          "NRVENDAREST" => $nrvendarest,
          "NRCOMANDA"   => $nrcomanda
        ];
    }

    private function getBranch($token) {
        $branch = $this->orderService->getBranch($token);

        if(empty($branch)) 
            throw new ParamsException("Nao foi encontrado filial associada ao token: $token.");
            
        return $branch;
    }

    private function getDefaultParams($cdFilial) {
        return $this->orderService->getDefaultParams($cdFilial);
    }

    private function getConsumer($cdcliente, $cdconsumidor) {
        return $this->consumerService->getConsumer($cdcliente, $cdconsumidor);
    }

    private function saveVendarest($cdFilial, $cdloja, $nrmesa, $cdvendedor, $cdoperador, $cdcliente, $cdconsumidor, $sgestado) {
        return $this->orderService->saveVendarest($cdFilial, $cdloja, $nrmesa, $cdvendedor, $cdoperador, $cdcliente, $cdconsumidor, $sgestado);
    }

    private function saveComandaven($cdFilial, $nrvendarest, $nrcomandaext, $cdloja, $idstcomanda, $deliveryFee, $nrpedmobile, $consumerData, $dsObsComanda, $consumerIdentifier, $dlvMethod) {
        return $this->orderService->saveComandaven($cdFilial, $nrvendarest, $nrcomandaext, $cdloja, $idstcomanda, $deliveryFee, $nrpedmobile, $consumerData, $dsObsComanda, $consumerIdentifier, $dlvMethod);
    }
    
    private function validateDuplication($defaultBranch, $orderId){
        return $this->orderService->validateDuplication($defaultBranch, $orderId);
    }

    private function saveMovcaixadlv($deliveryParams, $orderDetails, $cdFilial, $nrvendarest, $charges, $orderPaidInCash, $cdcliente, $cdconsumidor, $payments) {
        // $paymentCode = $this->paymentService->getPaymentCode();
        
        foreach($payments as $payment){
            if( $payment["code"] == "card" && empty($payment["cardSelected"]) ){
                $paymentCode = $this->paymentService->getPaymentCode();
            }
            elseif($payment["code"] == "card" && !empty($payment["cardSelected"])){
                $paymentCode = $this->paymentService->getReceiptType($cdFilial, "ANT", $payment["externalId"]);
            }
            elseif ($payment['code'] ==  'money'){
                $paymentCode = $this->paymentService->getMoneyReceiptCode();
            }    
            $this->paymentService->saveMovcaixadlv($deliveryParams, $orderDetails, $cdFilial, $nrvendarest, $charges, $paymentCode, $cdcliente, $cdconsumidor);
        }
    }

    private function acceptOrder($orderId) {
        
        $generalToken   = $this->util->getAccessToken();
        // $url         = self::API_ANT_BASE_URL."v1/order/order/confirm/?auth_token=$authToken&order_id=$orderId";
        $url         = self::API_ANT_BASE_URL."partnerauth/order/accept/$orderId";
        $body        = json_encode([]);
        
        $header = [
            "Content-type: application/json",
            "Authorization: " . $generalToken
        ];
    
        $response = $this->util->executeWebService($url, "POST", $header, $body);

        return json_decode($response, true);
    }

    private function rejectOrder($orderId) {
        $cancellationId = 922;
        // $reason         = "NAO FOI POSSIVEL REALIZAR INTEGRACAO";
        $generalToken      = $this->util->getAccessToken();
        // $url            = self::API_ANT_BASE_URL."v1/order/order/cancel/" . urlencode("?auth_token=$authToken&order_id=$orderId&reason_id=$cancellationId&reason=$reason");
        $url         = self::API_ANT_BASE_URL."partnerauth/order/cancel/$orderId";
        $body           = json_encode(["justification"=>"NAO FOI POSSIVEL REALIZAR INTEGRACAO"]);
        
        $header = [
            "Content-type: application/json",
            "Authorization: " . $generalToken
        ];
    
        $response = $this->util->executeWebService($url, "POST", $header, $body);

        return json_decode($response, true);
    }

    private function getVersion() {
        return json_decode(file_get_contents(__DIR__ . "/../../version.json"), true);
    }

    public function getConsumerAddress($formattedAddress) {
        
        $URL = self::GEOCODE_URL . "&address="  . urlencode($formattedAddress);

        $request = json_decode($this->util->executeWebService($URL, "GET", [], []), true);
        return $this->consumerService->getConsumerAddress($request['results'][0]);
    }
    
    public function cancelOrder(Request $request, Response $response) {
        try {

            
            $body = $request->getParsedBody();
            $orderId = $body['anotaAiData']['id'];

            $this->debugLog($body);

            $this->orderService->cancelOrder($orderId);
            

            $orderDetails = $this->getOrderDetailsById($orderId);
            
            $fileName = ((!empty($orderDetails["CDFILIAL"])) && (!empty($orderId))) ? "{$orderDetails['CDFILIAL']}-{$orderId}-{$this->util->getCurrentDateTime()}.json" : ("INDEFINIDO-" . $this->util->getCurrentDateTime()) . ".json";
            // $fileName     = $orderDetails["CDFILIAL"] . "-" . $orderDetails["NRCOMANDAEXT"] . "-". date("c") . ".json";

            $this->util->writeLog('cancelled', $fileName, $body);
        } catch(CancelException $e) {
            $errorMessage = $e->getMessage();
        } catch(\Exception $e) {
            $errorMessage = $e->getMessage();
        } ;
        if (!empty($errorMessage)) {
            $this->util->writeLog('cancelled_error', date("d-m-Y H:i:s") . '.txt', $errorMessage);
            return $response->withStatus(500);
        }

        return $response->withStatus(200);
    }
    
    public function getOrderDetailsById($orderId){
        return $this->orderService->getOrderDetailsById($orderId);
    }
    
    public function shareDiscount($initialDiscount, &$products) {

        $idxToApplyDiscount   = [];
        $totalDiscount        = $initialDiscount;
        $discountLeft         = $initialDiscount;
        $productsTotalValue   = "0.00";
        $discountGivenShare   = "0.00";
        $maxValueItem         = "0.00";
        $maxValueIdx          = -1;
        
        foreach($products as &$product) {
            $product['VRDESCCOMVEN']  = 0;
            $product['discountShare'] = "0.00";
        }
        
        if(isset($product))
            unset($product);

        foreach($products as $i => $product) {
            if((bccomp($product["VRPRECCOMVEN"], "0.01", 2) == 1) && empty($product["reedem_id"])) { 
                $productsTotalValue = bcadd($productsTotalValue, $product["VRPRECCOMVEN"], 2);
                array_push($idxToApplyDiscount, $i);
            }
        }

        foreach($idxToApplyDiscount as $idxWithDiscount) {
            $discountShare = bcdiv((bcmul($products[$idxWithDiscount]["VRPRECCOMVEN"], "100.00", 2)), $productsTotalValue, 2);
            $products[$idxWithDiscount]["discountShare"] = $discountShare;

            $discountGivenShare = bcadd($discountGivenShare, $discountShare, 2);
        }

        foreach($products as $i => $product) {
            if(bccomp($product['discountShare'], $maxValueItem, 2) == 1) {
                $maxValueIdx  = $i;
                $maxValueItem = $product['discountShare'];
            }            
        }

        if($maxValueIdx != -1) {
            $products[$maxValueIdx]["discountShare"] = bcadd($products[$maxValueIdx]["discountShare"], bcsub("100.00", $discountGivenShare, 2), 2);
        }
    
        foreach($idxToApplyDiscount as $idxWithDiscount) {
        
            $itemDiscount = bcdiv((bcmul($products[$idxWithDiscount]["discountShare"], $totalDiscount, 2)), "100.00", 2);
            $products[$idxWithDiscount]["VRDESCCOMVEN"] = $itemDiscount;

            $discountLeft = bcsub($discountLeft, $itemDiscount, 2);
        }

        if($maxValueIdx != -1) {
            $products[$maxValueIdx]["VRDESCCOMVEN"] = bcadd($products[$maxValueIdx]["VRDESCCOMVEN"], $discountLeft, 2);
            $discountLeft = "0.00";
        }

        return $products;
    }
    
}